@@ -147,23 +147,23 @@ ns_new_resolve(JSContext *context,
  * identify the prototype as an object of our class with NULL private
  * data.
  */
-static JSBool
-ns_constructor(JSContext *context,
-               JSObject  *obj,
-               uintN      argc,
-               jsval     *argv,
-               jsval     *retval)
+GJS_NATIVE_CONSTRUCTOR_DECLARE(ns)
 {
+    GJS_NATIVE_CONSTRUCTOR_VARIABLES(ns)
     Ns *priv;
 
+    GJS_NATIVE_CONSTRUCTOR_PRELUDE(ns);
+
     priv = g_slice_new0(Ns);
 
     GJS_INC_COUNTER(ns);
 
-    g_assert(priv_from_js(context, obj) == NULL);
-    JS_SetPrivate(context, obj, priv);
+    g_assert(priv_from_js(context, object) == NULL);
+    JS_SetPrivate(context, object, priv);
 
-    gjs_debug_lifecycle(GJS_DEBUG_GNAMESPACE, "ns constructor, obj %p priv %p", obj, priv);
+    gjs_debug_lifecycle(GJS_DEBUG_GNAMESPACE, "ns constructor, obj %p priv %p", object, priv);
+
+    GJS_NATIVE_CONSTRUCTOR_FINISH(ns);
 
     return JS_TRUE;
 }
@@ -247,7 +247,7 @@ ns_new(JSContext    *context,
                                   * none - just name the prototype like
                                   * Math - rarely correct)
                                   */
-                                 ns_constructor,
+                                 gjs_ns_constructor,
                                  /* number of constructor args */
                                  0,
                                  /* props of prototype */
