@@ -32,7 +32,8 @@
 #include "object.h"
 #include "boxed.h"
 #include "union.h"
-#include <gjs/gjs.h>
+#include <gjs/gjs-module.h>
+#include <gjs/compat.h>
 
 #include <girepository.h>
 
@@ -53,6 +54,7 @@ closure_marshal(GClosure        *closure,
                 gpointer         invocation_hint,
                 gpointer         marshal_data)
 {
+    JSRuntime *runtime;
     JSContext *context;
     int argc;
     jsval *argv;
@@ -64,12 +66,13 @@ closure_marshal(GClosure        *closure,
                       "Marshal closure %p",
                       closure);
 
-    context = gjs_closure_get_context(closure);
-    if (context == NULL) {
+    if (!gjs_closure_is_valid(closure)) {
         /* We were destroyed; become a no-op */
         return;
     }
 
+    runtime = gjs_closure_get_runtime(closure);
+    context = gjs_runtime_get_current_context(runtime);
     JS_BeginRequest(context);
 
     argc = n_param_values;
@@ -78,7 +81,7 @@ closure_marshal(GClosure        *closure,
 
     gjs_set_values(context, argv, argc, JSVAL_VOID);
     gjs_root_value_locations(context, argv, argc);
-    JS_AddRoot(context, &rval);
+    JS_AddValueRoot(context, &rval);
 
     if (marshal_data) {
         /* we are used for a signal handler */
@@ -138,7 +141,7 @@ closure_marshal(GClosure        *closure,
 
  cleanup:
     gjs_unroot_value_locations(context, argv, argc);
-    JS_RemoveRoot(context, &rval);
+    JS_RemoveValueRoot(context, &rval);
     JS_EndRequest(context);
 }
 
@@ -678,7 +681,7 @@ gjs_value_from_g_value(JSContext    *context,
     return gjs_value_from_g_value_internal(context, value_p, gvalue, FALSE);
 }
 
-__attribute__((constructor)) void
+__attribute__((constructor)) static void
 _gjs_value_init_types(void)
 {
     _array_type = g_type_from_name("GArray");
