@@ -24,10 +24,12 @@
 #include <config.h>
 
 #include "context.h"
-#include "context-jsapi.h"
 #include "importer.h"
 #include "jsapi-util.h"
 #include "profiler.h"
+#include "native.h"
+#include "byteArray.h"
+#include "compat.h"
 
 #include <util/log.h>
 #include <util/error.h>
@@ -62,7 +64,6 @@ struct _GjsContext {
     char **search_path;
 
     unsigned int we_own_runtime : 1;
-    unsigned int is_load_context : 1;
 };
 
 struct _GjsContextClass {
@@ -82,8 +83,7 @@ static int signals[LAST_SIGNAL];
 enum {
     PROP_0,
     PROP_SEARCH_PATH,
-    PROP_RUNTIME,
-    PROP_IS_LOAD_CONTEXT
+    PROP_RUNTIME
 };
 
 
@@ -93,11 +93,10 @@ static GList *all_contexts = NULL;
 
 static JSBool
 gjs_log(JSContext *context,
-        JSObject  *obj,
         uintN      argc,
-        jsval     *argv,
-        jsval     *retval)
+        jsval     *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
     char *s;
     JSExceptionState *exc_state;
     JSString *jstr;
@@ -132,16 +131,16 @@ gjs_log(JSContext *context,
     g_free(s);
 
     JS_EndRequest(context);
+    JS_SET_RVAL(context, vp, JSVAL_VOID);
     return JS_TRUE;
 }
 
 static JSBool
 gjs_log_error(JSContext *context,
-                 JSObject  *obj,
-                 uintN      argc,
-                 jsval     *argv,
-                 jsval     *retval)
+              uintN      argc,
+              jsval     *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
     char *s;
     JSExceptionState *exc_state;
     JSString *jstr;
@@ -181,6 +180,7 @@ gjs_log_error(JSContext *context,
     g_free(s);
 
     JS_EndRequest(context);
+    JS_SET_RVAL(context, vp, JSVAL_VOID);
     return JS_TRUE;
 }
 
@@ -197,7 +197,7 @@ gjs_print_parse_args(JSContext *context,
     JS_BeginRequest(context);
 
     str = g_string_new("");
-    JS_EnterLocalRootScope(context);
+    (void)JS_EnterLocalRootScope(context);
     for (n = 0; n < argc; ++n) {
         JSExceptionState *exc_state;
         JSString *jstr;
@@ -243,11 +243,10 @@ gjs_print_parse_args(JSContext *context,
 
 static JSBool
 gjs_print(JSContext *context,
-          JSObject  *obj,
           uintN      argc,
-          jsval     *argv,
-          jsval     *retval)
+          jsval     *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
     char *buffer;
 
     if (!gjs_print_parse_args(context, argc, argv, &buffer)) {
@@ -257,16 +256,16 @@ gjs_print(JSContext *context,
     g_print("%s\n", buffer);
     g_free(buffer);
 
+    JS_SET_RVAL(context, vp, JSVAL_VOID);
     return JS_TRUE;
 }
 
 static JSBool
 gjs_printerr(JSContext *context,
-             JSObject  *obj,
              uintN      argc,
-             jsval     *argv,
-             jsval     *retval)
+             jsval     *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
     char *buffer;
 
     if (!gjs_print_parse_args(context, argc, argv, &buffer)) {
@@ -276,16 +275,10 @@ gjs_printerr(JSContext *context,
     g_printerr("%s\n", buffer);
     g_free(buffer);
 
+    JS_SET_RVAL(context, vp, JSVAL_VOID);
     return JS_TRUE;
 }
 
-static JSClass global_class = {
-    "GjsGlobal", JSCLASS_GLOBAL_FLAGS,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
-    JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
-    JSCLASS_NO_OPTIONAL_MEMBERS
-};
-
 static void
 gjs_context_init(GjsContext *js_context)
 {
@@ -329,9 +322,7 @@ gjs_context_class_init(GjsContextClass *klass)
                                  FALSE,
                                  G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 
-    g_object_class_install_property(object_class,
-                                    PROP_IS_LOAD_CONTEXT,
-                                    pspec);
+    gjs_register_native_module("byteArray", gjs_define_byte_array_stuff, 0);
 }
 
 static void
@@ -347,46 +338,27 @@ gjs_context_dispose(GObject *object)
     }
 
     if (js_context->global != NULL) {
-        JS_BeginRequest(js_context->context);
-        JS_RemoveRoot(js_context->context, &js_context->global);
-        JS_EndRequest(js_context->context);
         js_context->global = NULL;
     }
 
     if (js_context->context != NULL) {
 
         gjs_debug(GJS_DEBUG_CONTEXT,
-                  "Destroying JS context%s",
-                  js_context->is_load_context ? " (load context)" : "");
+                  "Destroying JS context");
 
+        if (js_context->we_own_runtime)
+            gjs_runtime_set_default_context(js_context->runtime, NULL);
         JS_DestroyContext(js_context->context);
         js_context->context = NULL;
     }
 
     if (js_context->runtime != NULL) {
         if (js_context->we_own_runtime) {
-            /* Avoid keeping JSContext with a dangling pointer to the
-             * runtime.
+            /* Cleans up data as well as destroying the runtime. Foreign
+             * contexts aren't supported at the moment; if we supported
+             * them, then the data simply would't get cleaned up.
              */
-            gjs_runtime_clear_call_context(js_context->runtime);
-            gjs_runtime_clear_load_context(js_context->runtime);
-
-            gjs_debug(GJS_DEBUG_CONTEXT,
-                      "Destroying JS runtime");
-
-            JS_DestroyRuntime(js_context->runtime);
-
-            /* finalize the dataset from jsapi-util.c ...  for
-             * "foreign" runtimes this just never happens for
-             * now... we do this after the runtime itself is destroyed
-             * because we might have finalizers run by
-             * JS_DestroyRuntime() that rely on data we've set on the
-             * runtime, such as the dynamic class structs.
-             */
-            gjs_debug(GJS_DEBUG_CONTEXT,
-                      "Destroying any remaining dataset items on runtime");
-
-            g_dataset_destroy(js_context->runtime);
+            gjs_runtime_destroy(js_context->runtime);
         }
         js_context->runtime = NULL;
     }
@@ -550,6 +522,7 @@ gjs_context_constructor (GType                  type,
             gjs_fatal("Failed to create javascript runtime");
         JS_SetGCParameter(js_context->runtime, JSGC_MAX_BYTES, 0xffffffff);
         js_context->we_own_runtime = TRUE;
+        gjs_runtime_init(js_context->runtime);
     }
 
     js_context->context = JS_NewContext(js_context->runtime, 8192 /* stack chunk size */);
@@ -598,13 +571,9 @@ gjs_context_constructor (GType                  type,
         JS_SetVersion(js_context->context, OUR_JS_VERSION);
     }
 
-    js_context->global = JS_NewObject(js_context->context, &global_class, NULL, NULL);
-    if (js_context->global == NULL)
-        gjs_fatal("Failed to create javascript global object");
-
-    /* Sets global object and adds builtins to it */
-    if (!JS_InitStandardClasses(js_context->context, js_context->global))
-        gjs_fatal("Failed to init standard javascript classes");
+    if (!gjs_init_context_standard(js_context->context))
+        gjs_fatal("Failed to initialize context");
+    js_context->global = JS_GetGlobalObject(js_context->context);
 
     if (!JS_DefineProperty(js_context->context, js_context->global,
                            "window", OBJECT_TO_JSVAL(js_context->global),
@@ -612,63 +581,62 @@ gjs_context_constructor (GType                  type,
                            JSPROP_READONLY | JSPROP_PERMANENT))
         gjs_fatal("No memory to export global object as 'window'");
 
-    /* this is probably not necessary, having it as global object in
-     * context already roots it presumably? Could not find where it
-     * does in a quick glance through spidermonkey source though.
-     */
-    if (!JS_AddRoot(js_context->context, &js_context->global))
-        gjs_fatal("No memory to add global object as GC root");
-
     /* Define a global function called log() */
     if (!JS_DefineFunction(js_context->context, js_context->global,
                            "log",
-                           gjs_log,
-                           1, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_log,
+                           1, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         gjs_fatal("Failed to define log function");
 
     if (!JS_DefineFunction(js_context->context, js_context->global,
                            "logError",
-                           gjs_log_error,
-                           2, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_log_error,
+                           2, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         gjs_fatal("Failed to define logError function");
 
     /* Define global functions called print() and printerr() */
     if (!JS_DefineFunction(js_context->context, js_context->global,
                            "print",
-                           gjs_print,
-                           3, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_print,
+                           3, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         gjs_fatal("Failed to define print function");
     if (!JS_DefineFunction(js_context->context, js_context->global,
                            "printerr",
-                           gjs_printerr,
-                           4, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_printerr,
+                           4, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         gjs_fatal("Failed to define printerr function");
 
-    /* If we created the root importer in the load context,
-     * there would be infinite recursion since the load context
-     * is a GjsContext
+    /* We need to know what the default context is, since it's the context whose
+     * global object is used to load imported JS modules. We currently say that
+     * it's the context of the runtime's owner, but if we needed to support
+     * externally created runtimes, we could define it in some other fashion.
      */
-    if (!js_context->is_load_context) {
-        /* We create the global-to-runtime root importer with the
-         * passed-in search path. If someone else already created
-         * the root importer, this is a no-op.
-         */
-        if (!gjs_create_root_importer(js_context->runtime,
-                                      js_context->search_path ?
-                                      (const char**) js_context->search_path :
-                                      NULL,
-                                      TRUE))
-            gjs_fatal("Failed to create root importer");
-
-        /* Now copy the global root importer (which we just created,
-         * if it didn't exist) to our global object
-         */
-        if (!gjs_define_root_importer(js_context->context,
-                                      js_context->global,
-                                      "imports"))
-            gjs_fatal("Failed to point 'imports' property at root importer");
+    if (js_context->we_own_runtime) {
+        gjs_runtime_set_default_context(js_context->runtime, js_context->context);
+    } else {
+        if (gjs_runtime_get_default_context(js_context->runtime) == NULL)
+            gjs_fatal("GjsContext created for a runtime not owned by GJS");
     }
 
+    /* We create the global-to-runtime root importer with the
+     * passed-in search path. If someone else already created
+     * the root importer, this is a no-op.
+     */
+    if (!gjs_create_root_importer(js_context->context,
+                                  js_context->search_path ?
+                                  (const char**) js_context->search_path :
+                                  NULL,
+                                  TRUE))
+        gjs_fatal("Failed to create root importer");
+
+    /* Now copy the global root importer (which we just created,
+     * if it didn't exist) to our global object
+     */
+    if (!gjs_define_root_importer(js_context->context,
+                                  js_context->global,
+                                  "imports"))
+        gjs_fatal("Failed to point 'imports' property at root importer");
+
     if (js_context->we_own_runtime) {
         js_context->profiler = gjs_profiler_new(js_context->runtime);
     }
@@ -693,10 +661,6 @@ gjs_context_get_property (GObject     *object,
     js_context = GJS_CONTEXT (object);
 
     switch (prop_id) {
-    case PROP_IS_LOAD_CONTEXT:
-        g_value_set_boolean(value, js_context->is_load_context);
-        break;
-
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -720,9 +684,6 @@ gjs_context_set_property (GObject      *object,
     case PROP_RUNTIME:
         js_context->runtime = g_value_get_pointer(value);
         break;
-    case PROP_IS_LOAD_CONTEXT:
-        js_context->is_load_context = g_value_get_boolean(value);
-        break;
 
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -779,12 +740,6 @@ gjs_context_get_native_context (GjsContext *js_context)
     return js_context->context;
 }
 
-gboolean
-gjs_context_is_load_context(GjsContext *js_context)
-{
-    return js_context->is_load_context;
-}
-
 gboolean
 gjs_context_eval(GjsContext *js_context,
                  const char   *script,
@@ -831,6 +786,7 @@ gjs_context_eval(GjsContext *js_context,
     /* JS_EvaluateScript requires a request even though it sort of seems like
      * it means we're always in a request?
      */
+    gjs_runtime_push_context(js_context->runtime, js_context->context);
     JS_BeginRequest(js_context->context);
 
     retval = JSVAL_VOID;
@@ -895,9 +851,10 @@ gjs_context_eval(GjsContext *js_context,
         }
     }
 
-    g_object_unref(G_OBJECT(js_context));
-
     JS_EndRequest(js_context->context);
+    gjs_runtime_pop_context(js_context->runtime);
+
+    g_object_unref(G_OBJECT(js_context));
 
     return success;
 }
@@ -958,3 +915,34 @@ gjs_context_define_string_array(GjsContext  *js_context,
     return TRUE;
 }
 
+#if GJS_BUILD_TESTS
+
+void
+gjstest_test_func_gjs_context_construct_destroy(void)
+{
+    GjsContext *context;
+
+    /* Construct twice just to possibly a case where global state from
+     * the first leaks.
+     */
+    context = gjs_context_new ();
+    g_object_unref (context);
+
+    context = gjs_context_new ();
+    g_object_unref (context);
+}
+
+void
+gjstest_test_func_gjs_context_construct_eval(void)
+{
+    GjsContext *context;
+    int estatus;
+    GError *error = NULL;
+
+    context = gjs_context_new ();
+    if (!gjs_context_eval (context, "1+1", -1, "<input>", &estatus, &error))
+        g_error ("%s", error->message);
+    g_object_unref (context);
+}
+
+#endif /* GJS_BUILD_TESTS */
