@@ -25,8 +25,9 @@
 #include "dbus-exports.h"
 #include "dbus-values.h"
 
-#include "../gjs/gjs.h"
+#include <gjs/gjs-module.h>
 #include "../gi/closure.h"
+#include <gjs/compat.h>
 
 #include <util/log.h>
 #include <gjs-dbus/dbus.h>
@@ -104,7 +105,7 @@ gjs_js_push_current_message(DBusMessage *message)
 }
 
 void
-gjs_js_pop_current_message()
+gjs_js_pop_current_message(void)
 {
     g_assert(_gjs_current_dbus_messages != NULL);
 
@@ -252,7 +253,10 @@ complete_call(JSContext   *context,
     g_assert(ret_values != NULL);
 
     array_length = gjs_rooted_array_get_length(context, ret_values);
-    if (array_length == 1) {
+    if (array_length == 0) {
+        /* the callback expects to be called with callback(undefined, null) */
+        *retval = JSVAL_VOID;
+    } else if (array_length == 1) {
         /* If the array only has one element return that element alone */
         *retval = gjs_rooted_array_get(context,
                                           ret_values,
@@ -274,6 +278,18 @@ complete_call(JSContext   *context,
     return JS_TRUE;
 }
 
+static JSContext *
+get_callback_context(GClosure *closure)
+{
+    JSRuntime *runtime;
+
+    if (!gjs_closure_is_valid(closure))
+        return NULL;
+
+    runtime = gjs_closure_get_runtime(closure);
+    return gjs_runtime_get_current_context(runtime);
+}
+
 static void
 pending_notify(DBusPendingCall *pending,
                void            *user_data)
@@ -287,7 +303,7 @@ pending_notify(DBusPendingCall *pending,
 
     closure = user_data;
 
-    context = gjs_closure_get_context(closure);
+    context = get_callback_context(closure);
 
     gjs_debug(GJS_DEBUG_DBUS,
               "Notified of reply to async call closure %p context %p",
@@ -349,11 +365,11 @@ pending_free_closure(void *data)
 /* Args are bus_name, object_path, iface, method, out signature, in signature, args, and callback to get returned value */
 static JSBool
 gjs_js_dbus_call_async(JSContext  *context,
-                       JSObject   *obj,
                        uintN       argc,
-                       jsval      *argv,
-                       jsval      *retval)
+                       jsval      *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
+    JSObject *obj = JS_THIS_OBJECT(context, vp);
     GClosure *closure;
     DBusMessage *message;
     DBusPendingCall *pending;
@@ -632,7 +648,7 @@ signal_handler_callback(DBusConnection *connection,
         return;
     }
 
-    context = gjs_closure_get_context(handler->closure);
+    context = get_callback_context(handler->closure);
 
     if (!context) {
         /* The runtime is gone */
@@ -675,11 +691,11 @@ signal_handler_callback(DBusConnection *connection,
 /* Args are bus_name, object_path, iface, signal, and callback */
 static JSBool
 gjs_js_dbus_watch_signal(JSContext  *context,
-                         JSObject   *obj,
                          uintN       argc,
-                         jsval      *argv,
-                         jsval      *retval)
+                         jsval      *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
+    JSObject *obj = JS_THIS_OBJECT(context, vp);
     const char *bus_name;
     const char *object_path;
     const char *iface;
@@ -732,7 +748,7 @@ gjs_js_dbus_watch_signal(JSContext  *context,
      * ref to the SignalHandler
      */
 
-    *retval = INT_TO_JSVAL(id);
+    JS_SET_RVAL(context, vp, INT_TO_JSVAL(id));
 
     return JS_TRUE;
 }
@@ -740,11 +756,11 @@ gjs_js_dbus_watch_signal(JSContext  *context,
 /* Args are handler id */
 static JSBool
 gjs_js_dbus_unwatch_signal_by_id(JSContext  *context,
-                                 JSObject   *obj,
                                  uintN       argc,
-                                 jsval      *argv,
-                                 jsval      *retval)
+                                 jsval      *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
+    JSObject *obj = JS_THIS_OBJECT(context, vp);
     int id;
     DBusBusType bus_type;
 
@@ -766,11 +782,11 @@ gjs_js_dbus_unwatch_signal_by_id(JSContext  *context,
 /* Args are bus_name, object_path, iface, signal, and callback */
 static JSBool
 gjs_js_dbus_unwatch_signal(JSContext  *context,
-                           JSObject   *obj,
                            uintN       argc,
-                           jsval      *argv,
-                           jsval      *retval)
+                           jsval      *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
+    JSObject *obj = JS_THIS_OBJECT(context, vp);
     const char *bus_name;
     const char *object_path;
     const char *iface;
@@ -834,11 +850,11 @@ gjs_js_dbus_unwatch_signal(JSContext  *context,
 /* Args are object_path, iface, signal, arguments signature, arguments */
 static JSBool
 gjs_js_dbus_emit_signal(JSContext  *context,
-                        JSObject   *obj,
                         uintN       argc,
-                        jsval      *argv,
-                        jsval      *retval)
+                        jsval      *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
+    JSObject *obj = JS_THIS_OBJECT(context, vp);
     DBusConnection *bus_connection;
     DBusMessage *message;
     DBusMessageIter arg_iter;
@@ -910,11 +926,10 @@ gjs_js_dbus_emit_signal(JSContext  *context,
  * to ensure that a signal has been sent before proceeding. */
 static JSBool
 gjs_js_dbus_flush(JSContext  *context,
-                        JSObject   *obj,
-                        uintN       argc,
-                        jsval      *argv,
-                        jsval      *retval)
+                  uintN       argc,
+                  jsval      *vp)
 {
+    JSObject *obj = JS_THIS_OBJECT(context, vp);
     DBusConnection *bus_connection;
     DBusBusType bus_type;
 
@@ -941,17 +956,18 @@ gjs_js_dbus_flush(JSContext  *context,
 /* Args are bus_name, object_path, iface, method, out signature, in signature, args */
 static JSBool
 gjs_js_dbus_call(JSContext  *context,
-                 JSObject   *obj,
                  uintN       argc,
-                 jsval      *argv,
-                 jsval      *retval)
+                 jsval      *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
+    JSObject *obj = JS_THIS_OBJECT(context, vp);
     DBusMessage *message;
     DBusError derror;
     DBusMessage *reply;
     JSBool result;
     DBusConnection *bus_connection;
     DBusBusType bus_type;
+    jsval retval;
 
     if (argc < 8) {
         gjs_throw(context, "Not enough args, need bus name, object path, interface, method, out signature, in signature, autostart flag, and args");
@@ -975,7 +991,11 @@ gjs_js_dbus_call(JSContext  *context,
     /* The retval is (we hope) rooted by jsapi when it invokes the
      * native function
      */
-    result = complete_call(context, retval, reply, &derror);
+    retval = JSVAL_NULL;
+    JS_AddValueRoot(context, &retval);
+    result = complete_call(context, &retval, reply, &derror);
+    if (result)
+        JS_SET_RVAL(context, vp, retval);
 
     if (reply)
         dbus_message_unref(reply);
@@ -1003,7 +1023,7 @@ on_name_acquired(DBusConnection *connection,
 
     owner = data;
 
-    context = gjs_closure_get_context(owner->acquired_closure);
+    context = get_callback_context(owner->acquired_closure);
     if (context == NULL) {
         gjs_debug(GJS_DEBUG_DBUS,
                   "Closure destroyed before we could notify name acquired");
@@ -1015,16 +1035,16 @@ on_name_acquired(DBusConnection *connection,
     argc = 1;
 
     argv[0] = STRING_TO_JSVAL(JS_NewStringCopyZ(context, name));
-    JS_AddRoot(context, &argv[0]);
+    JS_AddValueRoot(context, &argv[0]);
 
     rval = JSVAL_VOID;
-    JS_AddRoot(context, &rval);
+    JS_AddValueRoot(context, &rval);
 
     gjs_closure_invoke(owner->acquired_closure,
                        argc, argv, &rval);
 
-    JS_RemoveRoot(context, &argv[0]);
-    JS_RemoveRoot(context, &rval);
+    JS_RemoveValueRoot(context, &argv[0]);
+    JS_RemoveValueRoot(context, &rval);
 
     JS_EndRequest(context);
 }
@@ -1042,7 +1062,7 @@ on_name_lost(DBusConnection *connection,
 
     owner = data;
 
-    context = gjs_closure_get_context(owner->lost_closure);
+    context = get_callback_context(owner->lost_closure);
     if (context == NULL) {
         gjs_debug(GJS_DEBUG_DBUS,
                   "Closure destroyed before we could notify name lost");
@@ -1054,16 +1074,16 @@ on_name_lost(DBusConnection *connection,
     argc = 1;
 
     argv[0] = STRING_TO_JSVAL(JS_NewStringCopyZ(context, name));
-    JS_AddRoot(context, &argv[0]);
+    JS_AddValueRoot(context, &argv[0]);
 
     rval = JSVAL_VOID;
-    JS_AddRoot(context, &rval);
+    JS_AddValueRoot(context, &rval);
 
     gjs_closure_invoke(owner->lost_closure,
                           argc, argv, &rval);
 
-    JS_RemoveRoot(context, &argv[0]);
-    JS_RemoveRoot(context, &rval);
+    JS_RemoveValueRoot(context, &argv[0]);
+    JS_RemoveValueRoot(context, &rval);
 
     JS_EndRequest(context);
 }
@@ -1092,11 +1112,11 @@ owner_closure_invalidated(gpointer  data,
 /* Args are bus_name, name type, acquired_func, lost_func */
 static JSBool
 gjs_js_dbus_acquire_name(JSContext  *context,
-                         JSObject   *obj,
                          uintN       argc,
-                         jsval      *argv,
-                         jsval      *retval)
+                         jsval      *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
+    JSObject *obj = JS_THIS_OBJECT(context, vp);
     const char *bus_name;
     JSObject *acquire_func;
     JSObject *lost_func;
@@ -1104,6 +1124,7 @@ gjs_js_dbus_acquire_name(JSContext  *context,
     DBusBusType bus_type;
     GjsDBusNameType name_type;
     unsigned int id;
+    jsval retval = JSVAL_VOID;
 
     if (argc < 4) {
         gjs_throw(context, "Not enough args, need bus name, name type, acquired_func, lost_func");
@@ -1168,10 +1189,11 @@ gjs_js_dbus_acquire_name(JSContext  *context,
                                &owner->funcs,
                                owner);
 
-    if (!JS_NewNumberValue(context, (jsdouble)id, retval)) {
+    if (!JS_NewNumberValue(context, (jsdouble)id, &retval)) {
         gjs_throw(context, "Could not convert name owner id to jsval");
         return JS_FALSE;
     }
+    JS_SET_RVAL(context, vp, retval);
 
     return JS_TRUE;
 }
@@ -1179,11 +1201,11 @@ gjs_js_dbus_acquire_name(JSContext  *context,
 /* Args are name owner monitor id */
 static JSBool
 gjs_js_dbus_release_name_by_id (JSContext  *context,
-                                JSObject   *obj,
                                 uintN       argc,
-                                jsval      *argv,
-                                jsval      *retval)
+                                jsval      *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
+    JSObject *obj = JS_THIS_OBJECT(context, vp);
     DBusBusType bus_type;
     unsigned int id;
 
@@ -1199,6 +1221,7 @@ gjs_js_dbus_release_name_by_id (JSContext  *context,
 
     gjs_dbus_release_name_by_id(bus_type,
                                 id);
+    JS_SET_RVAL(context, vp, JSVAL_VOID);
     return JS_TRUE;
 }
 
@@ -1223,7 +1246,7 @@ on_name_appeared(DBusConnection *connection,
 
     watcher = data;
 
-    context = gjs_closure_get_context(watcher->appeared_closure);
+    context = get_callback_context(watcher->appeared_closure);
     if (context == NULL) {
         gjs_debug(GJS_DEBUG_DBUS,
                   "Closure destroyed before we could notify name appeared");
@@ -1241,12 +1264,12 @@ on_name_appeared(DBusConnection *connection,
     argv[1] = STRING_TO_JSVAL(JS_NewStringCopyZ(context, owner_unique_name));
 
     rval = JSVAL_VOID;
-    JS_AddRoot(context, &rval);
+    JS_AddValueRoot(context, &rval);
 
     gjs_closure_invoke(watcher->appeared_closure,
                           argc, argv, &rval);
 
-    JS_RemoveRoot(context, &rval);
+    JS_RemoveValueRoot(context, &rval);
     gjs_unroot_value_locations(context, argv, argc);
 
     JS_EndRequest(context);
@@ -1266,7 +1289,7 @@ on_name_vanished(DBusConnection *connection,
 
     watcher = data;
 
-    context = gjs_closure_get_context(watcher->vanished_closure);
+    context = get_callback_context(watcher->vanished_closure);
     if (context == NULL) {
         gjs_debug(GJS_DEBUG_DBUS,
                   "Closure destroyed before we could notify name vanished");
@@ -1284,12 +1307,12 @@ on_name_vanished(DBusConnection *connection,
     argv[1] = STRING_TO_JSVAL(JS_NewStringCopyZ(context, owner_unique_name));
 
     rval = JSVAL_VOID;
-    JS_AddRoot(context, &rval);
+    JS_AddValueRoot(context, &rval);
 
     gjs_closure_invoke(watcher->vanished_closure,
                           argc, argv, &rval);
 
-    JS_RemoveRoot(context, &rval);
+    JS_RemoveValueRoot(context, &rval);
     gjs_unroot_value_locations(context, argv, argc);
 
     JS_EndRequest(context);
@@ -1326,11 +1349,11 @@ watch_closure_invalidated(gpointer  data,
 /* Args are bus_name, start_if_not_found, appeared_func, vanished_func */
 static JSBool
 gjs_js_dbus_watch_name(JSContext  *context,
-                       JSObject   *obj,
                        uintN       argc,
-                       jsval      *argv,
-                       jsval      *retval)
+                       jsval      *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
+    JSObject *obj = JS_THIS_OBJECT(context, vp);
     const char *bus_name;
     JSBool start_if_not_found;
     JSObject *appeared_func;
@@ -1400,6 +1423,7 @@ gjs_js_dbus_watch_name(JSContext  *context,
                         &watch_name_funcs,
                         watcher);
 
+    JS_SET_RVAL(context, vp, JSVAL_VOID);
     return JS_TRUE;
 }
 
@@ -1409,7 +1433,7 @@ gjs_js_dbus_watch_name(JSContext  *context,
 static JSBool
 unique_name_getter(JSContext  *context,
                    JSObject   *obj,
-                   jsval       id,
+                   jsid        id,
                    jsval      *value_p)
 {
     const char *name;
@@ -1419,7 +1443,8 @@ unique_name_getter(JSContext  *context,
     if (!get_bus_type_from_object(context, obj, &bus_type))
         return JS_FALSE;
 
-    name = gjs_string_get_ascii(id);
+    if (!gjs_get_string_id(context, id, &name))
+        return JS_FALSE;
 
     gjs_debug_jsprop(GJS_DEBUG_DBUS, "Get prop '%s' on dbus object", name);
 
@@ -1444,11 +1469,10 @@ unique_name_getter(JSContext  *context,
 
 static JSBool
 gjs_js_dbus_signature_length(JSContext  *context,
-                             JSObject   *obj,
                              uintN       argc,
-                             jsval      *argv,
-                             jsval      *retval)
+                             jsval      *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
     const char *signature;
     DBusSignatureIter iter;
     int length = 0;
@@ -1478,18 +1502,18 @@ gjs_js_dbus_signature_length(JSContext  *context,
     } while (dbus_signature_iter_next(&iter));
 
  out:
-    *retval = INT_TO_JSVAL(length);
+    JS_SET_RVAL(context, vp, INT_TO_JSVAL(length));
 
     return JS_TRUE;
 }
 
 static JSBool
 gjs_js_dbus_start_service(JSContext  *context,
-                          JSObject   *obj,
                           uintN       argc,
-                          jsval      *argv,
-                          jsval      *retval)
+                          jsval      *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
+    JSObject *obj = JS_THIS_OBJECT(context, vp);
     const char     *name;
     DBusBusType     bus_type;
     DBusConnection *bus_connection;
@@ -1519,7 +1543,7 @@ gjs_js_dbus_start_service(JSContext  *context,
 static JSBool
 gjs_js_dbus_get_machine_id(JSContext *context,
                            JSObject  *obj,
-                           jsval      key,
+                           jsid       key,
                            jsval     *value)
 {
     char *machine_id;
@@ -1543,11 +1567,10 @@ gjs_js_dbus_get_machine_id(JSContext *context,
 
 static JSBool
 gjs_js_dbus_get_current_message_context(JSContext  *context,
-                                        JSObject   *obj,
                                         uintN       argc,
-                                        jsval      *argv,
-                                        jsval      *retval)
+                                        jsval      *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
     const char *sender;
     JSString *sender_str;
     JSObject *context_obj;
@@ -1559,7 +1582,7 @@ gjs_js_dbus_get_current_message_context(JSContext  *context,
         return JS_FALSE;
 
     if (!_gjs_current_dbus_messages) {
-        *retval = JSVAL_NULL;
+        JS_SET_RVAL(context, vp, JSVAL_NULL);
         return JS_TRUE;
     }
 
@@ -1570,7 +1593,7 @@ gjs_js_dbus_get_current_message_context(JSContext  *context,
         return JS_FALSE;
 
     context_val = OBJECT_TO_JSVAL(context_obj);
-    JS_AddRoot(context, &context_val);
+    JS_AddValueRoot(context, &context_val);
 
     sender = dbus_message_get_sender(current_message);
     if (sender)
@@ -1592,10 +1615,10 @@ gjs_js_dbus_get_current_message_context(JSContext  *context,
         goto out;
 
     result = JS_TRUE;
-    *retval = context_val;
+    JS_SET_RVAL(context, vp, context_val);
 
 out:
-    JS_RemoveRoot(context, &context_val);
+    JS_RemoveValueRoot(context, &context_val);
     return result;
 }
 
@@ -1612,7 +1635,7 @@ define_bus_proto(JSContext *context,
     result = JS_FALSE;
 
     bus_proto_val = JSVAL_VOID;
-    JS_AddRoot(context, &bus_proto_val);
+    JS_AddValueRoot(context, &bus_proto_val);
 
     bus_proto_obj = JS_ConstructObject(context, NULL, NULL, NULL);
     if (bus_proto_obj == NULL)
@@ -1630,69 +1653,69 @@ define_bus_proto(JSContext *context,
 
     if (!JS_DefineFunction(context, bus_proto_obj,
                            "call",
-                           gjs_js_dbus_call,
-                           8, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_js_dbus_call,
+                           8, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         goto out;
 
     if (!JS_DefineFunction(context, bus_proto_obj,
                            "call_async",
-                           gjs_js_dbus_call_async,
-                           9, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_js_dbus_call_async,
+                           9, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         goto out;
 
     if (!JS_DefineFunction(context, bus_proto_obj,
                            "acquire_name",
-                           gjs_js_dbus_acquire_name,
-                           3, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_js_dbus_acquire_name,
+                           3, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         goto out;
 
     if (!JS_DefineFunction(context, bus_proto_obj,
                            "release_name_by_id",
-                           gjs_js_dbus_release_name_by_id,
-                           1, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_js_dbus_release_name_by_id,
+                           1, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         goto out;
 
 
     if (!JS_DefineFunction(context, bus_proto_obj,
                            "watch_name",
-                           gjs_js_dbus_watch_name,
-                           4, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_js_dbus_watch_name,
+                           4, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         goto out;
 
     if (!JS_DefineFunction(context, bus_proto_obj,
                            "watch_signal",
-                           gjs_js_dbus_watch_signal,
-                           5, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_js_dbus_watch_signal,
+                           5, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         goto out;
 
     if (!JS_DefineFunction(context, bus_proto_obj,
                            "unwatch_signal_by_id",
-                           gjs_js_dbus_unwatch_signal_by_id,
-                           1, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_js_dbus_unwatch_signal_by_id,
+                           1, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         goto out;
 
     if (!JS_DefineFunction(context, bus_proto_obj,
                            "unwatch_signal",
-                           gjs_js_dbus_unwatch_signal,
-                           5, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_js_dbus_unwatch_signal,
+                           5, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         goto out;
 
     if (!JS_DefineFunction(context, bus_proto_obj,
                            "emit_signal",
-                           gjs_js_dbus_emit_signal,
-                           3, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_js_dbus_emit_signal,
+                           3, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         goto out;
 
     if (!JS_DefineFunction(context, bus_proto_obj,
                            "flush",
-                           gjs_js_dbus_flush,
-                           0, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_js_dbus_flush,
+                           0, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         goto out;
 
     if (!JS_DefineFunction(context, bus_proto_obj,
                            "start_service",
-                           gjs_js_dbus_start_service,
-                           1, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_js_dbus_start_service,
+                           1, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         goto out;
 
     /* Add the bus proto object inside the passed in module object */
@@ -1706,7 +1729,7 @@ define_bus_proto(JSContext *context,
     result = JS_TRUE;
 
  out:
-    JS_RemoveRoot(context, &bus_proto_val);
+    JS_RemoveValueRoot(context, &bus_proto_val);
 
     return result;
 }
@@ -1731,11 +1754,18 @@ define_bus_object(JSContext  *context,
     result = JS_FALSE;
 
     bus_val = JSVAL_VOID;
-    JS_AddRoot(context, &bus_val);
+    JS_AddValueRoot(context, &bus_val);
 
-    bus_obj = JS_ConstructObject(context, NULL, proto_obj, NULL);
+    bus_obj = JS_ConstructObject(context, NULL, NULL, NULL);
     if (bus_obj == NULL)
         goto out;
+    /* We need to use a separate call to SetPrototype to work
+     * around a SpiderMonkey bug where with clasp=NULL, the
+     * parent and proto arguments to JS_ConstructObject are
+     * lost.
+     * https://bugzilla.mozilla.org/show_bug.cgi?id=599651
+     */
+    JS_SetPrototype(context, bus_obj, proto_obj);
 
     bus_val = OBJECT_TO_JSVAL(bus_obj);
 
@@ -1767,7 +1797,7 @@ define_bus_object(JSContext  *context,
     result = JS_TRUE;
 
  out:
-    JS_RemoveRoot(context, &bus_val);
+    JS_RemoveValueRoot(context, &bus_val);
 
     return result;
 }
@@ -1783,42 +1813,42 @@ gjs_js_define_dbus_stuff(JSContext      *context,
 
     if (!JS_DefineFunction(context, module_obj,
                            "signatureLength",
-                           gjs_js_dbus_signature_length,
-                           1, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_js_dbus_signature_length,
+                           1, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         return JS_FALSE;
 
     if (!JS_DefineProperty(context, module_obj,
                            "BUS_SESSION",
                            INT_TO_JSVAL(DBUS_BUS_SESSION),
                            NULL, NULL,
-                           GJS_MODULE_PROP_FLAGS))
+                           GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         return JS_FALSE;
 
     if (!JS_DefineProperty(context, module_obj,
                            "BUS_SYSTEM",
                            INT_TO_JSVAL(DBUS_BUS_SYSTEM),
                            NULL, NULL,
-                           GJS_MODULE_PROP_FLAGS))
+                           GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         return JS_FALSE;
 
     if (!JS_DefineProperty(context, module_obj,
                            "BUS_STARTER",
                            INT_TO_JSVAL(DBUS_BUS_STARTER),
                            NULL, NULL,
-                           GJS_MODULE_PROP_FLAGS))
+                           GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         return JS_FALSE;
 
     if (!JS_DefineProperty(context, module_obj,
                            "localMachineID",
                            JSVAL_VOID,
                            gjs_js_dbus_get_machine_id, NULL,
-                           GJS_MODULE_PROP_FLAGS))
+                           GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         return JS_FALSE;
 
     if (!JS_DefineFunction(context, module_obj,
                            "getCurrentMessageContext",
-                           gjs_js_dbus_get_current_message_context,
-                           0, GJS_MODULE_PROP_FLAGS))
+                           (JSNative)gjs_js_dbus_get_current_message_context,
+                           0, GJS_MODULE_PROP_FLAGS | JSFUN_FAST_NATIVE))
         return JS_FALSE;
 
     /* Define both the session and system objects */
