@@ -1,4 +1,5 @@
-const Everything = imports.gi.Everything;
+// This used to be called "Everything"
+const Everything = imports.gi.Regress;
 if (!('assertEquals' in this)) { /* allow running this test standalone */
     imports.lang.copyPublicProperties(imports.jsUnit, this);
     gjstestRun = function() { return imports.jsUnit.gjstestRun(window); };
@@ -76,6 +77,7 @@ function testBoxedCopyConstructor()
     // Copy an object from another object of the same type, shortcuts to the boxed copy
     let copy = new Everything.TestSimpleBoxedA(simple_boxed);
 
+    assertTrue(copy instanceof Everything.TestSimpleBoxedA);
     assertEquals(42, copy.some_int);
     assertEquals(43, copy.some_int8);
     assertEquals(42.5, copy.some_double);
